﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CommonPrinter;

namespace PrintWindowDemo
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
        }

        private void menuItemExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private enum StretchMode
        {
            NONE,       // No stretch image
            WIDTH,      // Fit image in printer page width
            FULL,       // Fit image in printer page
        }

        // New PIvoke declaration of StartDoc
        [System.Runtime.InteropServices.DllImport("coredll.dll", SetLastError=true, EntryPoint="StartDocW")]
		public static extern Int32 StartDoc(IntPtr hdc, ref TCommonPrinter.DOCINFO docInfo);

        private void PrintWindow(Control control, StretchMode stretchMode)
        {
            TCommonPrinter printer;
            IntPtr printerDC;
            IntPtr windowDC;         

            printer = new TCommonPrinter();
            printerDC = IntPtr.Zero;
            windowDC = IntPtr.Zero;

            try
            {
                // Open connection with printer
                printerDC = printer.CreatePrinterDC(true);
                if (printerDC == IntPtr.Zero)
                    throw new Exception("Failed to connect to printer.");
                
                // Get window dc
                if (control == null)
                    windowDC = TCommonGDI.GetDC(IntPtr.Zero);
                else
                    windowDC = TCommonGDI.GetDC(control.Handle);

                // Get printer graphics page size
                Size printerPage;
                printerPage = new Size(printer.PageWidth, printer.PageHeight);

                // Get dimensions of the window
                Rectangle clientRect;
                if (control == null)
                {
                    clientRect = Screen.PrimaryScreen.Bounds;                    
                }
                else
                {
                    clientRect = new Rectangle(0, 0, control.Width, control.Height);                    
                }
                              
                Size windowSize;
                windowSize = new Size(clientRect.Width, clientRect.Height);
                int dx = windowSize.Width;
                int dy = windowSize.Height;

                // Calculate bitmap new dimensions
                if (stretchMode != StretchMode.NONE) // i.e. != NONE
                {                    
                    if (dx != printerPage.Width)
                    {
                        dy = TCommonWin32.MulDiv(printerPage.Width, windowSize.Height, windowSize.Width);
                        dx = printerPage.Width;
                    }

                    if (stretchMode == StretchMode.FULL && dy > printerPage.Height) // total stretch?
                    {
                        dx = TCommonWin32.MulDiv(printerPage.Height, dx, dy);
                        dy = printerPage.Height;
                    }
                }

                // Here from CF2.0 we can use "Graphics" object 
                TCommonGDI.SelectObject(printerDC, TCommonGDI.GetStockObject(TCommonGDI.NULL_PEN));
                TCommonGDI.SelectObject(printerDC, TCommonGDI.GetStockObject(TCommonGDI.WHITE_BRUSH));

                int srcOffset = 0;
                int dstHeight = TCommonWin32.MulDiv(windowSize.Height - srcOffset, dx, windowSize.Width);

                TCommonPrinter.DOCINFO docInfo = new TCommonPrinter.DOCINFO();
                docInfo.cbSize = System.Runtime.InteropServices.Marshal.SizeOf(docInfo);
                // There is much more easy and understandable way to invoke StartDoc.
                // You just need to define new PInvoke of StartDoc
                /*                              
                System.Runtime.InteropServices.GCHandle bfr = new System.Runtime.InteropServices.GCHandle();
                IntPtr ptr = bfr.AddrOfPinnedObject();
                StartDoc(hdc, ptr);                
                */                
                if (StartDoc(printerDC, ref docInfo) < 0)
                    throw new Exception("Failed to start document.");

                while (srcOffset < windowSize.Height)
                {
                    if (TCommonPrinter.StartPage(printerDC) < 0)
                    {
                        TCommonPrinter.AbortDoc(printerDC);
                        throw new Exception("Failed to start page.");
                    }
                    
                    TCommonGDI.Rectangle(printerDC, 0, 0, printerPage.Width, printerPage.Height);
                    TCommonGDI.TransparentImage(
                            printerDC,
                            0,
                            0,
                            dx,
                            dstHeight,
                            windowDC,
                            0,
                            srcOffset,
                            windowSize.Width,
                            windowSize.Height - srcOffset,
                            TCommonGDI.CLR_INVALID);

                    srcOffset += TCommonWin32.MulDiv(printerPage.Height, windowSize.Width, dx);
                    dstHeight = TCommonWin32.MulDiv(windowSize.Height - srcOffset, dx, windowSize.Width);

                    if (TCommonPrinter.EndPage(printerDC) < 0)
                    {
                        TCommonPrinter.AbortDoc(printerDC);
                        throw new Exception("Failed to end page.");
                    }

                }

                if (TCommonPrinter.EndDoc(printerDC) < 0)
                    throw new Exception("Failed to end document.");
            }
            finally
            {                
                printer.DeletePrinterDC(printerDC);

                if (control == null)
                    TCommonGDI.ReleaseDC(IntPtr.Zero, windowDC);
                else
                    TCommonGDI.ReleaseDC(control.Handle, windowDC);
            }
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            comboBoxStretch.SelectedIndex = 0;
            comboBoxWindow.SelectedIndex = 0;
        }

        private void menuItemPrint_Click(object sender, EventArgs e)
        {
            StretchMode stretchMode = StretchMode.NONE;

            switch (comboBoxStretch.SelectedIndex)
            {
                case 0:
                    stretchMode = StretchMode.NONE;
                    break;
                case 1:
                    stretchMode = StretchMode.WIDTH;
                    break;
                case 2:               
                    stretchMode = StretchMode.FULL;
                    break;
            }

            Control control = null;
            switch (comboBoxWindow.SelectedIndex)
            {
                case 0:
                    control = (Control)this;
                    break;
            }

            try
            {
                PrintWindow(control, stretchMode);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Print Window", MessageBoxButtons.OK, MessageBoxIcon.Hand, MessageBoxDefaultButton.Button1);
            }
        }
    }
}